<!doctype html>
<html>
    <head>
        <meta charset="UTF-8">
        <title>tty-share</title>
        <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/nf-sauce-code-pro@2.1.3/nf-font.min.css">
    </head>
    <body>
        <div id="terminal"></div>
        <div id="settings"></div>
        <script type="text/javascript">
            window.ttyInitialData = {
                wsPath: {{.WSPath}}
            }
        </script>
        <script src="{{.PathPrefix}}/static/tty-share.js"></script>
    </body>
</html>

