<!-- This is taken from: https://colorlib.com/wp/free-404-error-page-templates/ -->
<!DOCTYPE html>

<html lang="en">
<head>
    <link rel="stylesheet" type="text/css" href="{{.PathPrefix}}/static/404.css">
</head>
<body>
    <section id="not-found">
        <div id="title">Something's wrong</div>
        <div class="circles">
            <p>404
                <br>
                <small>Page not found</small>
            </p>
            <span class="circle big"></span>
            <span class="circle med"></span>
            <span class="circle small"></span>
        </div>
    </section>
</body>
</html>
